#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>
#include <fstream>
#include <string>
#include <float.h>
#include <math.h>

#include "randomgen.h"

#define RELEASE
#ifndef NUM
#define NUM 2000
#endif
//#define NUM 10

static const float UPPER = 0.9;
static const float LOWER = 0.1;
int width = 0;
// #ifndef RADIUS
// #define RADIUS 0.05
// #endif

using namespace std;

float RandomEqual(float min, float max)
{
	float x = (float)rand()/RAND_MAX;
	return x*(max-min)+min;
}

float RandomPeak(float min, float max, int dim)
{
	float sum = 0.0;
	for (int d=0; d<dim; d++)
		sum += RandomEqual(0, 1);
	sum /= dim;
	return sum*(max-min)+min;
}

float RandomNormal(float med, float var)
{
	return RandomPeak(med-var, med+var, 12);
}

float loc(float v, float s)
{
	float d = v / s;
	long i = (long)d;
	if (d > i)
	{
		i ++;
	}

	return (float)i * s;
}

void GenerateDataEqually(float** center, int Dimensions, int Count, int DupRatio)
{
	float step = 1.0 / ((float)Count / (float)DupRatio);

	for (int i=0; i<Count; i++)
	{
		float x[Dimensions];
		for (int d=0; d<Dimensions; d++)
		{
			x[d] = RandomEqual(LOWER, UPPER);
			if (DupRatio == 1)
			{
				center[i][d] = x[d];
			}
			else
			{
				center[i][d] = loc(x[d], step);
			}
		}
	}
}

void GenerateDataCorrelated(float** center, int Dimensions, int Count,
		int DupRatio)
{
	float step = 1.0 / ((float)Count / (float)DupRatio);

	float x[Dimensions];
	for (long i=0; i<Count; i++)
	{
		again: float v = RandomPeak(LOWER, UPPER, Dimensions);
		for (int d=0; d<Dimensions; d++)
			x[d] = v;
		float l = v<=(LOWER+UPPER)/2 ? v : UPPER-v;
		for (int d=0; d<Dimensions; d++)
		{
			float h = RandomNormal(LOWER, l);
			x[d] += h;
			x[(d+1)%Dimensions] -= h;
		}
		for (int d=0; d<Dimensions; d++)
			if (x[d]<LOWER || x[d]>=UPPER)
				goto again;

		for (int d=0; d<Dimensions; d++)
		{
			if (DupRatio == 1)
			{
				center[i][d] = x[d];
			}
			else
			{
				center[i][d] = loc(x[d], step);
			}
		}
	}
}

void GenerateDataAnticorrelated(float** center, int Dimensions, int Count,
		int DupRatio)
{
	float step = 1.0 / ((float)Count / (float)DupRatio);

	float x[Dimensions];
	for (long i=0; i<Count; i++)
	{
		again: float v = RandomNormal((LOWER+UPPER)/2, (LOWER+UPPER)/4);
		for (int d=0; d<Dimensions; d++)
			x[d] = v;
		float l = v<=(LOWER+UPPER)/2 ? v : UPPER-v;
		for (int d=0; d<Dimensions; d++)
		{
			float h = RandomEqual(-l, l);
			x[d] += h;
			x[(d+1)%Dimensions] -= h;
		}
		for (int d=0; d<Dimensions; d++)
			if (x[d]<LOWER || x[d]>=UPPER)
				goto again;
		for (int d=0; d<Dimensions; d++)
		{
			if (DupRatio == 1)
			{
				center[i][d] = x[d];
			}
			else
			{
				center[i][d] = loc(x[d], step);
			}
		}
	}
}

void GenerateCenter(float** center, int Dimensions, char Distribution, int Count)
{
	if (Count <= 0)
	{
		cout << "Amount should be greater than 0"<< endl;
		return;
	}
	if (Dimensions < 2)
	{
		cout << "Dimension should be greater than 2"<< endl;
		return;
	}
	switch (Distribution)
	{
	case 'E':
	case 'e':
		Distribution = 'E';
		break;
	case 'C':
	case 'c':
		Distribution = 'C';
		break;
	case 'A':
	case 'a':
		Distribution = 'A';
		break;
	default:
		cout << "Unknown data distribution error."<< endl;
		return;
	}

	switch (Distribution)
	{
	case 'E':
		//      srand((unsigned)time(NULL));
		GenerateDataEqually(center, Dimensions , Count, 1);
		break;
	case 'C':
		//      srand((unsigned)time(NULL));
		GenerateDataCorrelated(center, Dimensions , Count, 1);
		break;
	case 'A':
		//      srand((unsigned)time(NULL));
		GenerateDataAnticorrelated(center, Dimensions , Count, 1);
		break;
	}
}

void GenerateGroup(ostream& out, long groupID, float* center, float radius,
		int dimension, long min_group_size, long max_group_size)
{
	//generate points, normal distribution
	long groupSize = (long)RandomEqual(min_group_size, max_group_size);
	out << groupID << " "<< groupSize << endl;

	//generate points in group
	CRandomGen ran;
	float** normal = new float*[dimension];
	float tmp1;
	float tmp2;
	for (int i = 0; i < dimension; ++i)
	{
		normal[i] = new float[groupSize];
		for (int j = 0; j < groupSize - 1; j = j + 2)
		{
			do
			{
				ran.nextNormal(0.0, 1.0, -FLT_MAX, FLT_MAX, tmp1, tmp2);
			} while (tmp1 > 3|| tmp1< -3 || tmp2 > 3 || tmp2 < -3);
			normal[i][j] = tmp1;
			normal[i][j+1] = tmp2;
		}
		if (groupSize % 2== 1)
		{
			do
			{
				ran.nextNormal(0.0, 1.0, -FLT_MAX, FLT_MAX, tmp1, tmp2);
			} while (tmp1 > 3|| tmp1< -3 || tmp2 > 3 || tmp2 < -3);
			normal[i][groupSize - 1] = tmp1;
		}
		for (long j = 0; j < groupSize; ++j)
		{
			normal[i][j] *= radius / 3;
			normal[i][j] += center[i];
		}
	}

	for (long i = 0; i < groupSize; ++i)
	{
		for (int j = 0; j < dimension; ++j)
		{
			out << normal[j][i] << " ";
		}
		out << endl;
	}
    
	for (int i = 0; i < dimension; ++i)
	{
		delete[] normal[i];
	}
	delete[] normal;
}

void generate(long min_group_size, long max_group_size, int dimension,
		long startNo, long groupNo, char dist, ostream& out)
{
	//  srand(time(NULL));

	float** center;
	center = new float*[groupNo];
	for (long i = 0; i < groupNo; ++i)
	{
		center[i] = new float[dimension];
	}
	GenerateCenter(center, dimension, dist, groupNo);

	float radius[groupNo];
	long id[groupNo];

	float ran_max = -FLT_MAX;
	float ran_min= FLT_MAX;
	CRandomGen ran;
	for (long i = 0; i < groupNo / 2; ++i)
	{
		ran.nextNormal(0.0, 1.0, -FLT_MAX, FLT_MAX, radius[2*i], radius[2*i+1]);
		ran_max = max(ran_max, radius[2*i]);
		ran_max = max(ran_max, radius[2*i+1]);
		ran_min = min(ran_min, radius[2*i]);
		ran_min = min(ran_min, radius[2*i+1]);
	}
	float r = max(ran_max, -ran_min);
	for (long i = 0; i < groupNo; ++i)
	{
		radius[i] = radius[i]*width / (40000 * r)+ (float)width/40000;
		cout << "radius is:  " << radius[i] << endl;
	}

	//debug
	//   for(long i = 0; i < groupNo; ++i)
	//   {
	//     cout << radius[i] << endl;
	//   }
	//   float mean = 0.0;
	//   float delta = 0.0;
	//   for (long i = 0; i < groupNo; ++i)
	//   {
	//     mean += radius[i];
	//   }
	//   mean = mean / groupNo;
	//   for (long i = 0; i < groupNo; ++i)
	//   {
	//     delta += (radius[i] - mean) * (radius[i] - mean);
	//   }
	//   delta = sqrt(delta / groupNo);
	//   cout << mean << " " << delta << endl;

	for (long i = 0; i < groupNo; ++i)
	{
		GenerateGroup(out, i+startNo, center[i], radius[i], dimension,
				min_group_size, max_group_size);
	}

	for (long i = 0; i < groupNo; ++i)
	{
		delete[] center[i];
	}
	delete[] center;
}

void generateNormal(long numOfPoints, int DIMENSION,  ostream& out)
{
	float ran_max = -FLT_MAX;
	float ran_min= FLT_MAX;
	CRandomGen ran;
	long j=1;
	for (long i = 0; i < numOfPoints/2; ++i)
	{
		float x,y;
		float a,b;
		
		do
		{
			ran.nextNormal(0.0, 1.0, -FLT_MAX, FLT_MAX, x, a);
		} while (x > 3|| x< -3 || a > 3 || a < -3);
		
		do
		{
			ran.nextNormal(0.0, 1.0, -FLT_MAX, FLT_MAX, y, b);
		} while (y > 3|| y< -3 || b > 3 || b < -3);




		out <<  j << " " <<  (x+3)/6 << " " << (y+3)/6 << endl;
		j++;
		out <<  j << " " << (a+3)/6 << " " << (b+3)/6  << endl;
		j++;
	}


}

void generateCorrelated(long numOfPoints, int DIMENSION,  ostream& out)
{
	float step = 1.0 / ((float)numOfPoints);
	
	int DupRatio =1;

	float x[DIMENSION];
	for (long i=0; i<numOfPoints; i++)
	{
		out << i+1 << " ";
		again: float v = RandomPeak(LOWER, UPPER, DIMENSION);
		for (int d=0; d<DIMENSION; d++)
			x[d] = v;
		float l = v<=(LOWER+UPPER)/2 ? v : UPPER-v;
		for (int d=0; d<DIMENSION; d++)
		{
			float h = RandomNormal(LOWER, l);
			x[d] += h;
			x[(d+1)%DIMENSION] -= h;
		}
		for (int d=0; d<DIMENSION; d++)
			if (x[d]<LOWER || x[d]>=UPPER)
				goto again;

		for (int d=0; d<DIMENSION; d++)
		{
			if (DupRatio == 1)
			{
				out << x[d] << " ";
			}
			else
			{
				out <<  loc(x[d], step) << " ";
			}
		}
		out << endl;

	}
}

void generateAnticorrelated(long numOfPoints, int Dimensions, ostream& out)
{
	int DupRatio =1;
	float step = 1.0 / ((float)numOfPoints / (float)DupRatio);

	float x[Dimensions];
	for (long i=0; i<numOfPoints; i++)
	{
		out << i + 1 << " ";
		again: float v = RandomNormal((LOWER+UPPER)/2, (LOWER+UPPER)/4);
		for (int d=0; d<Dimensions; d++)
			x[d] = v;
		float l = v<=(LOWER+UPPER)/2 ? v : UPPER-v;
		for (int d=0; d<Dimensions; d++)
		{
			float h = RandomEqual(-l, l);
			x[d] += h;
			x[(d+1)%Dimensions] -= h;
		}
		for (int d=0; d<Dimensions; d++)
			if (x[d]<LOWER || x[d]>=UPPER)
				goto again;
		for (int d=0; d<Dimensions; d++)
		{
			if (DupRatio == 1)
			{
				out << x[d] << " " ;
			}
			else
			{
				out <<  loc(x[d], step) << " " ;
				
			}
		}
		out << endl;

	}
}


void generateUniform(long numOfPoints, int DIMENSION,  ostream& out)
{
	float ran_max = -FLT_MAX;
	float ran_min= FLT_MAX;
	
	for (long i = 0; i < numOfPoints; ++i)
	{
		out << i+1 << "	";
		for(int j=0; j<DIMENSION; j++)
		{
			
			double x;
			
					x= RandomEqual(0,1);
			out << x << "	";
			//out <<  x << "	" << y << endl;
		}
		out << endl;
	}


}


int main(int argc, char *argv[])
{
	srand(time(NULL));
#ifdef RELEASE
	long numOfPoints;
	int types;
	int overlap=-1;
	if(argc < 3)
	{
		cerr << "data generator" << endl
		<< "Usage: " << argv[0] << endl
		<< "  Number of points: " << endl //arg[1]
		<< "  distribution group centers (A(nticorrelated) | C(orrelated) | U(niform) | N(ormal) " << endl;
	}
	else
	{
		cout << "hello " << endl;
		numOfPoints = atol(argv[1]);
		int numD = atoi(argv[1]);
		 char f1[10];
		 char f2[10];
       		 sprintf(f1,"%d",numD);
		string filename = "../Raw_files/" + string(f1) + "_";
		
	cout << filename << endl;	
		
		
		switch(argv[2][0])
		{
			case 'c':
			case 'C':
			filename+= "corr";
			break;
			case 'u':
			case 'U':
			filename+= "unif";
			break;
			case 'a':
			case 'A':
			filename+= "anti";
			break;
			case 'n':
			case 'N':
			filename+= "norm";
			break;

			default:
			cerr<< "Unknown Distribution..." <<endl;
			exit(1);
		}
		fstream out;
		out.open(filename.c_str(), ios::out | ios::trunc);
		out.setf(ios::fixed, ios::floatfield);
		out.precision(15);


		switch(argv[2][0])
		{
			case 'c':
			case 'C':
			
			break;
			case 'u':
			case 'U':
			generateUniform(numOfPoints,2,out);
			break;
			case 'a':
			case 'A':
			generateAnticorrelated(numOfPoints,2, out);
			break;
			case 'n':
			case 'N':
			generateNormal(numOfPoints,2, out);
			break;

		}
		cout << "generated " << filename << endl;
	}
#endif
	
	return EXIT_SUCCESS;
}


